/* test routines for CRC-32

  Author:
    Eric Durbin
    Kentucky Cancer Registry
    University of Kentucky
    1998-12-15

  Status:
    Public Domain
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "crc32.h"

char  StdBuffer[18];
char *StdString = "123456789";

int StandardTest(void);
void PositionTest(void);

int main()
{
	StandardTest();
	PositionTest();
	return 0;
}

/* perform test with standard values:  
   string "123456789" should return CBF43926 */
int StandardTest()
{
	unsigned long CRCValue;
	unsigned long StdCRC32 = 0xCBF43926;

	/* standard test */
	printf("Testing CalcCRC32 with Standard String %s:\n", StdString);
	strcpy(StdBuffer, StdString);
    printf("STD CRC = %08lX\n", StdCRC32);
	CRCValue = CalcCRC32(StdBuffer, 17L, 10L, 8L);
	printf("CRCValue = %08lX\n", (unsigned long)CRCValue);
	if (CRCValue == StdCRC32)
		printf("Passed\n");
	else
		printf("%08lX does not equal %08lX:  Failed\n", CRCValue, StdCRC32);
	printf("\n");

	printf("Testing AssignCRC32, placing checksum at end of StdString:\n");
	AssignCRC32(StdBuffer, 17L, 10L, 8L);
	printf("%17.17s\n\n", StdBuffer);

	printf("Testing CompareCRC32 on above string:\n");
	if (CompareCRC32(StdBuffer, 17L, 10L, 8L) == 0)
		printf("Passed\n\n");
	else
		printf("Failed\n\n");

	printf("Testing CompareCRC32 after introducing error:\n");
	*StdBuffer = '2';
	printf("%s\n", StdBuffer);
	if (CompareCRC32(StdBuffer, 17L, 10L, 8L) == 0)
		printf("Passed\n\n");
	else
		printf("Failed\n\n");
	
	return 0;
}

/* test calculation and compare for each possible checksum postion
   in the standard test string 
*/
void PositionTest()
{
	int t, j;
	int	rv;

	for (t = 0; t < 2; t++) {
		if (t == 0)
			printf("Testing Position of Checksum Field in Standard String:\n");
		else
			printf("Testing Position of Checksum Field in Standard String with Random Error\n");
		for (j = 0; j < 10; j++) {
			/* blank buffer */
			memset(StdBuffer, '\0', sizeof(StdBuffer));
	
			/* copy first part of string */
			strncpy(StdBuffer, StdString, j);
	
			/* copy later part of string */
			strncpy(StdBuffer+j+8, StdString + j, 9-j);
	
			/* calculate and assign crc */
			AssignCRC32(StdBuffer, 17L, j+1, 8L);
	
			/* introduce random error on second pass */
			if (t == 1)
				/* assign random character to random position */
				StdBuffer[rand() % 17] = 'A' + rand() % 25;
	
			/* test crc 32 */
			rv = CompareCRC32(StdBuffer, 17L, j+1, 8L);
			printf("%s %s\n", StdBuffer, rv?"Failed":"Passed");
		}
		printf("\n");
	}
}
